@@ -14,7 +14,7 @@ if this_dir not in sys.path:
     sys.path.insert(0, this_dir) # needed for Py3
 
 from common_imports import etree, StringIO, BytesIO, HelperTestCase, fileInTestDir
-from common_imports import LargeFileLikeUnicode, doctest, make_doctest
+from common_imports import SillyFileLike, LargeFileLikeUnicode, doctest, make_doctest
 from common_imports import canonicalize, sorted, _str, _bytes
 
 print("")
@@ -430,6 +430,11 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertRaises(
             LookupError, self.etree.XMLParser, encoding="hopefully unknown")
 
+    def test_parser_encoding(self):
+        self.etree.XMLParser(encoding="ascii")
+        self.etree.XMLParser(encoding="utf-8")
+        self.etree.XMLParser(encoding="iso-8859-1")
+
     def test_elementtree_parser_target_type_error(self):
         assertEquals = self.assertEquals
         assertFalse  = self.assertFalse
@@ -525,6 +530,31 @@ class ETreeOnlyTestCase(HelperTestCase):
                            "data-ca", "end-a", "data-B", "end-root"],
                           events)
 
+    def test_parser_target_recover(self):
+        events = []
+        class Target(object):
+            def start(self, tag, attrib):
+                events.append("start-" + tag)
+            def end(self, tag):
+                events.append("end-" + tag)
+            def data(self, data):
+                events.append("data-" + data)
+            def close(self):
+                events.append("close")
+                return "DONE"
+
+        parser = self.etree.XMLParser(target=Target(),
+                                      recover=True)
+
+        parser.feed(_bytes('<root>A<a>ca</a>B</not-root>'))
+        done = parser.close()
+
+        self.assertEquals("DONE", done)
+        self.assertEquals(["start-root", "data-A", "start-a",
+                           "data-ca", "end-a", "data-B",
+                           "end-root", "close"],
+                          events)
+
     def test_iterwalk_tag(self):
         iterwalk = self.etree.iterwalk
         root = self.etree.XML(_bytes('<a><b><d/></b><c/></a>'))
@@ -642,6 +672,48 @@ class ETreeOnlyTestCase(HelperTestCase):
         root = tree.getroot()
         self.assertEquals(root.text, test_url)
 
+    def test_resolve_bytes_dtd(self):
+        parse = self.etree.parse
+        parser = self.etree.XMLParser(dtd_validation=True)
+        assertEqual = self.assertEqual
+        test_url = _str("__nosuch.dtd")
+
+        class MyResolver(self.etree.Resolver):
+            def resolve(self, url, id, context):
+                assertEqual(url, test_url)
+                return self.resolve_string(
+                    (_str('''<!ENTITY myentity "%s">
+                             <!ELEMENT doc ANY>''') % url).encode('utf-8'),
+                    context)
+
+        parser.resolvers.add(MyResolver())
+
+        xml = _str('<!DOCTYPE doc SYSTEM "%s"><doc>&myentity;</doc>') % test_url
+        tree = parse(StringIO(xml), parser)
+        root = tree.getroot()
+        self.assertEquals(root.text, test_url)
+
+    def test_resolve_filelike_dtd(self):
+        parse = self.etree.parse
+        parser = self.etree.XMLParser(dtd_validation=True)
+        assertEqual = self.assertEqual
+        test_url = _str("__nosuch.dtd")
+
+        class MyResolver(self.etree.Resolver):
+            def resolve(self, url, id, context):
+                assertEqual(url, test_url)
+                return self.resolve_file(
+                    SillyFileLike(
+                        _str('''<!ENTITY myentity "%s">
+                        <!ELEMENT doc ANY>''') % url), context)
+
+        parser.resolvers.add(MyResolver())
+
+        xml = _str('<!DOCTYPE doc SYSTEM "%s"><doc>&myentity;</doc>') % test_url
+        tree = parse(StringIO(xml), parser)
+        root = tree.getroot()
+        self.assertEquals(root.text, test_url)
+
     def test_resolve_filename_dtd(self):
         parse = self.etree.parse
         parser = self.etree.XMLParser(attribute_defaults=True)
@@ -664,6 +736,51 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(
             root[0].attrib, {'default': 'valueB'})
 
+    def test_resolve_filename_dtd_relative(self):
+        parse = self.etree.parse
+        parser = self.etree.XMLParser(attribute_defaults=True)
+        assertEqual = self.assertEqual
+        test_url = _str("__nosuch.dtd")
+
+        class MyResolver(self.etree.Resolver):
+            def resolve(self, url, id, context):
+                assertEqual(url, fileInTestDir(test_url))
+                return self.resolve_filename(
+                    fileInTestDir('test.dtd'), context)
+
+        parser.resolvers.add(MyResolver())
+
+        xml = _str('<!DOCTYPE a SYSTEM "%s"><a><b/></a>') % test_url
+        tree = parse(StringIO(xml), parser,
+                     base_url=fileInTestDir('__test.xml'))
+        root = tree.getroot()
+        self.assertEquals(
+            root.attrib,    {'default': 'valueA'})
+        self.assertEquals(
+            root[0].attrib, {'default': 'valueB'})
+
+    def test_resolve_file_dtd(self):
+        parse = self.etree.parse
+        parser = self.etree.XMLParser(attribute_defaults=True)
+        assertEqual = self.assertEqual
+        test_url = _str("__nosuch.dtd")
+
+        class MyResolver(self.etree.Resolver):
+            def resolve(self, url, id, context):
+                assertEqual(url, test_url)
+                return self.resolve_file(
+                    open(fileInTestDir('test.dtd'), 'rb'), context)
+
+        parser.resolvers.add(MyResolver())
+
+        xml = _str('<!DOCTYPE a SYSTEM "%s"><a><b/></a>') % test_url
+        tree = parse(StringIO(xml), parser)
+        root = tree.getroot()
+        self.assertEquals(
+            root.attrib,    {'default': 'valueA'})
+        self.assertEquals(
+            root[0].attrib, {'default': 'valueB'})
+
     def test_resolve_empty(self):
         parse = self.etree.parse
         parser = self.etree.XMLParser(load_dtd=True)
@@ -1961,6 +2078,24 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(docinfo.root_name,   'html')
         self.assertEquals(docinfo.doctype, '')
 
+    def test_docinfo_name_only(self):
+        etree = self.etree
+        xml = _bytes('<!DOCTYPE root><root></root>')
+        tree = etree.parse(BytesIO(xml))
+        docinfo = tree.docinfo
+        self.assertEquals(docinfo.encoding,    "UTF-8")
+        self.assertEquals(docinfo.xml_version, "1.0")
+        self.assertEquals(docinfo.public_id,   None)
+        self.assertEquals(docinfo.system_url,  None)
+        self.assertEquals(docinfo.root_name,   'root')
+        self.assertEquals(docinfo.doctype, '<!DOCTYPE root>')
+
+    def test_doctype_name_only_roundtrip(self):
+        etree = self.etree
+        xml = _bytes('<!DOCTYPE root>\n<root/>')
+        tree = etree.parse(BytesIO(xml))
+        self.assertEquals(xml, etree.tostring(tree))
+
     def test_xml_base(self):
         etree = self.etree
         root = etree.XML(_bytes("<root/>"), base_url="http://no/such/url")
